@@ -39,7 +39,6 @@ usage () {
 ALLINONE_PRESET="/etc/archboot/presets/allinone"
 ALLINONE_LTS_PRESET="/etc/archboot/presets/allinone-lts"
 TARBALL_HELPER="/usr/bin/archboot-tarball-helper.sh"
-USBIMAGE_HELPER="/usr/bin/archboot-usbimage-helper.sh"
 UPDATEISO_HELPER="/usr/bin/archboot-update-iso.sh"
 
 # change to english locale!
@@ -65,6 +64,12 @@ if ! [[ ${UID} -eq 0 ]]; then
 	exit 1
 fi
 
+### check for available loop devices in a container
+for i in $(seq 0 7); do
+    ! [[ -e /dev/loop$i ]] && mknod /dev/loop$i b 7 $i
+done
+! [[ -e /dev/loop-control ]] && mknod /dev/loop-control c 10 237
+
 if [[ "${TARBALL}" == "1" ]]; then
 	"${TARBALL_HELPER}" -c="${ALLINONE_PRESET}" -t="core-$(uname -m).tar"
 	"${TARBALL_HELPER}" -c="${ALLINONE_LTS_PRESET}" -t="core-lts-$(uname -m).tar"
@@ -194,15 +199,6 @@ _prepare_other_files() {
 	mkdir -p "${ALLINONE}/arch/"
 	mv "${CORE}/tmp"/*/arch/archboot.txt "${ALLINONE}/arch/"
 	
-	# copy in clamav db files
-	if [[ -d /var/lib/clamav && -x /usr/bin/freshclam ]]; then
-		mkdir -p "${ALLINONE}/clamav"
-		rm -f /var/lib/clamav/*
-		freshclam --user=root
-		cp /var/lib/clamav/{daily,main,bytecode}.cvd "${ALLINONE}/clamav/"
-		cp /var/lib/clamav/mirrors.dat "${ALLINONE}/clamav/"
-	fi
-	
 }
 
 _download_uefi_shell_tianocore() {
@@ -233,14 +229,12 @@ GUMEOF
 title    Arch Linux ${_UEFI_ARCH} Archboot
 linux    /boot/vmlinuz_${_UEFI_ARCH}
 initrd   /boot/initramfs_${_UEFI_ARCH}.img
-options  gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}
+options  gpt loglevel=7 efivars.pstore_disable=1 efi_pstore.pstore_disable=1 efi_no_storage_paranoia add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}
 GUMEOF
 	
 	cat << GUMEOF > "${ALLINONE}/loader/entries/archboot-${_UEFI_ARCH}-lts-efilinux.conf"
 title    Arch Linux LTS ${_UEFI_ARCH} Archboot via EFILINUX
-linux    /EFI/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi
-initrd   /boot/initramfs_${_UEFI_ARCH}.img
-options  -f \\boot\\vmlinuz_${_UEFI_ARCH}_lts gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}
+efi      /EFI/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi
 GUMEOF
 	
 	cat << GUMEOF > "${ALLINONE}/loader/entries/uefi-shell-${_UEFI_ARCH}-v2.conf"
@@ -261,8 +255,8 @@ GUMEOF
 	mkdir -p "${ALLINONE}/EFI/efilinux"
 	cp -f "/usr/lib/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi" "${ALLINONE}/EFI/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi"
 	
-	cat << EOF > "${ALLINONE}/EFI/efilinux/efilinux_._cfg_"
--f \\boot\\vmlinuz_${_UEFI_ARCH}_lts gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH} initrd=\\boot\\initramfs_${_UEFI_ARCH}.img
+	cat << EOF > "${ALLINONE}/EFI/efilinux/efilinux.cfg"
+-f \\boot\\vmlinuz_${_UEFI_ARCH}_lts gpt loglevel=7 efivars.pstore_disable=1 efi_pstore.pstore_disable=1 efi_no_storage_paranoia add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH} initrd=\\boot\\initramfs_${_UEFI_ARCH}.img
 EOF
 	
 }
@@ -306,7 +300,7 @@ menuentry "Arch Linux ${_UEFI_ARCH} Archboot" {
     icon /EFI/refind/icons/os_arch.icns
     loader /boot/vmlinuz_${_UEFI_ARCH}
     initrd /boot/initramfs_${_UEFI_ARCH}.img
-    options "gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}"
+    options "gpt loglevel=7 efivars.pstore_disable=1 efi_pstore.pstore_disable=1 efi_no_storage_paranoia add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}"
     ostype Linux
     graphics off
 }
@@ -314,8 +308,6 @@ menuentry "Arch Linux ${_UEFI_ARCH} Archboot" {
 menuentry "Arch Linux LTS ${_UEFI_ARCH} Archboot via EFILINUX" {
     icon /EFI/refind/icons/os_arch.icns
     loader /EFI/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi
-    initrd /boot/initramfs_${_UEFI_ARCH}.img
-    options "-f \\boot\\vmlinuz_${_UEFI_ARCH}_lts gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}"
     ostype Linux
     graphics off
 }
@@ -375,29 +367,64 @@ xorriso -as mkisofs \
         -isohybrid-mbr /usr/lib/syslinux/isohdpfx.bin \
         -output "${IMAGENAME}.iso" "${ALLINONE}/" &> "/tmp/archboot_allinone_xorriso.log"
 
-## cleanup isolinux and migrate to syslinux
-# echo "Generating ALLINONE IMG ..."
-# rm -f "${ALLINONE}/boot/syslinux/isolinux.bin"
-
-## Change parameters in boot.msg
-# sed -i -e "s/@@DATE@@/$(date)/g" -e "s/@@KERNEL@@/$KERNEL/g" -e "s/@@LTS_KERNEL@@/$LTS_KERNEL/g" -e "s/@@RELEASENAME@@/$RELEASENAME/g" -e "s/@@BOOTLOADER@@/SYSLINUX/g" "${ALLINONE}/boot/syslinux/boot.msg"
-
-# "${USBIMAGE_HELPER}" "${ALLINONE}" "${IMAGENAME}.img" > /dev/null 2>&1
-
+# create x86_64 iso, if not present
 if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-x86_64.iso" ]]; then
 	_REMOVE_i686="1" _REMOVE_x86_64="0" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
 	mv "${WD}/${IMAGENAME_OLD}-dual-updated-x86_64.iso" "${WD}/${IMAGENAME_OLD}-x86_64.iso"
 fi
 
+# create i686 iso, if not present
 if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-i686.iso" ]]; then
 	_REMOVE_i686="0" _REMOVE_x86_64="1" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
 	mv "${WD}/${IMAGENAME_OLD}-dual-updated-i686.iso" "${WD}/${IMAGENAME_OLD}-i686.iso"
 fi
 
+# create i686 ftp iso, if not present
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-i686-ftp.iso" ]]; then
+	_REMOVE_PACKAGES="1" _REMOVE_i686="0" _REMOVE_x86_64="1" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-i686-ftp.iso" "${WD}/${IMAGENAME_OLD}-i686-ftp.iso"
+fi
+
+# create dual iso with uefi cd boot support, if not present
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-dual-uefi.iso" ]]; then
+	_UPDATE_CD_UEFI="1" _REMOVE_i686="0" _REMOVE_x86_64="0" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-dual-uefi.iso" "${WD}/${IMAGENAME_OLD}-dual-uefi.iso"
+fi
+
+# create dual ftp iso with uefi cd boot support, if not present
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-dual-uefi-ftp.iso" ]]; then
+	_REMOVE_PACKAGES="1" _UPDATE_CD_UEFI="1" _REMOVE_i686="0" _REMOVE_x86_64="0" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-dual-uefi-ftp.iso" "${WD}/${IMAGENAME_OLD}-dual-uefi-ftp.iso"
+fi
+
+# create dual ftp iso, if not present
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-dual-ftp.iso" ]]; then
+	_REMOVE_PACKAGES="1" _REMOVE_i686="0" _REMOVE_x86_64="0" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-dual-ftp.iso" "${WD}/${IMAGENAME_OLD}-dual-ftp.iso"
+fi
+
+# create x86_64 iso with uefi cd boot support, if not present
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-x86_64-uefi.iso" ]]; then
+	_UPDATE_CD_UEFI="1" _REMOVE_i686="1" _REMOVE_x86_64="0" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-x86_64-uefi.iso" "${WD}/${IMAGENAME_OLD}-x86_64-uefi.iso"
+fi
+
+# create x86_64 ftp iso with uefi cd boot support, if not present
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-x86_64-uefi-ftp.iso" ]]; then
+	_REMOVE_PACKAGES="1" _UPDATE_CD_UEFI="1" _REMOVE_i686="1" _REMOVE_x86_64="0" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-x86_64-uefi-ftp.iso" "${WD}/${IMAGENAME_OLD}-x86_64-uefi-ftp.iso"
+fi
+
+# create x86_64 ftp iso, if not present
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-x86_64-ftp.iso" ]]; then
+	_REMOVE_PACKAGES="1" _REMOVE_i686="1" _REMOVE_x86_64="0" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-x86_64-ftp.iso" "${WD}/${IMAGENAME_OLD}-x86_64-ftp.iso"
+fi
+
 ## create sha256sums.txt
 cd "${WD}/"
 rm -f "${WD}/sha256sums.txt" || true
-sha256sum *.iso *.img > "${WD}/sha256sums.txt"
+sha256sum *.iso > "${WD}/sha256sums.txt"
 
 # cleanup
 rm -rf "${CORE}"
