@@ -1,10 +1,16 @@
 #!/usr/bin/env bash
 # Script for updating existing Archboot iso with newer UEFI shell, rEFInd, and /arch/setup script in the initramfs files
-# Contributed by "Keshav P R" <the.ridikulus.rat aatt geemmayil ddoott ccoomm>
+# Contributed by "Keshav Padram" <the ddoott ridikulus ddoott rat aatt geemmayil ddoott ccoomm>
+# change to english locale!
+export LANG="en_US"
 
 [[ -z "${_REMOVE_i686}" ]] && _REMOVE_i686="0"
 [[ -z "${_REMOVE_x86_64}" ]] && _REMOVE_x86_64="0"
 
+[[ -z "${_REMOVE_PACKAGES}" ]] && _REMOVE_PACKAGES="0"
+
+[[ -z "${_UPDATE_CD_UEFI}" ]] && _UPDATE_CD_UEFI="0"
+
 [[ -z "${_UPDATE_SETUP}" ]] && _UPDATE_SETUP="1"
 [[ -z "${_UPDATE_UEFI_SHELL}" ]] && _UPDATE_UEFI_SHELL="1"
 [[ -z "${_UPDATE_UEFI_GUMMIBOOT}" ]] && _UPDATE_UEFI_GUMMIBOOT="1"
@@ -35,14 +41,21 @@ _ARCHBOOT_ISO_EXT_DIR="$(mktemp -d /tmp/archboot_iso_ext.XXXXXXXXXX)"
 
 if [[ "${_REMOVE_x86_64}" != "1" ]] && [[ "${_REMOVE_i686}" != "1" ]]; then
 	_ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}-updated-dual"
+	[[ "${_UPDATE_CD_UEFI}" == "1" ]] && _ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}"-updated-dual-uefi
+	[[ "${_REMOVE_PACKAGES}" == "1" ]] && _ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}"-updated-dual-ftp
+	[[ "${_REMOVE_PACKAGES}" == "1" && "${_UPDATE_CD_UEFI}" == "1" ]] && _ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}"-updated-dual-uefi-ftp
 fi
 
 if [[ "${_REMOVE_x86_64}" != "1" ]] && [[ "${_REMOVE_i686}" == "1" ]]; then
 	_ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}-updated-x86_64"
+	[[ "${_UPDATE_CD_UEFI}" == "1" ]] && _ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}"-updated-x86_64-uefi
+	[[ "${_REMOVE_PACKAGES}" == "1" ]] && _ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}"-updated-x86_64-ftp
+        [[ "${_REMOVE_PACKAGES}" == "1" && "${_UPDATE_CD_UEFI}" == "1" ]] && _ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}"-updated-x86_64-uefi-ftp
 fi
 
 if [[ "${_REMOVE_x86_64}" == "1" ]] && [[ "${_REMOVE_i686}" != "1" ]]; then
 	_ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}-updated-i686"
+        [[ "${_REMOVE_PACKAGES}" == "1" ]] && _ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}"-updated-i686-ftp
 fi
 
 _ARCHBOOT_ISO_UPDATED_PATH="${_ARCHBOOT_ISO_WD}/${_ARCHBOOT_ISO_UPDATED_NAME}.iso"
@@ -71,6 +84,12 @@ if ! [[ ${UID} -eq 0 ]]; then
 	exit 1
 fi
 
+### check for available loop devices in a container
+for i in $(seq 0 7); do
+    ! [[ -e /dev/loop$i ]] && mknod /dev/loop$i b 7 $i
+done
+! [[ -e /dev/loop-control ]] && mknod /dev/loop-control c 10 237
+
 set -x
 
 ## Remove old files
@@ -82,7 +101,7 @@ echo
 
 ## Extract the archboot iso using bsdtar
 bsdtar -C "${_ARCHBOOT_ISO_EXT_DIR}/" -xf "${_ARCHBOOT_ISO_OLD_PATH}"
-# 7z x "${_ARCHBOOT_ISO_OLD_PATH}"
+# 7z x -o "${_ARCHBOOT_ISO_EXT_DIR}/" "${_ARCHBOOT_ISO_OLD_PATH}"
 echo
 
 rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/[BOOT]/" || true
@@ -311,14 +330,12 @@ GUMEOF
 title    Arch Linux ${_UEFI_ARCH} Archboot
 linux    /boot/vmlinuz_${_UEFI_ARCH}
 initrd   /boot/initramfs_${_UEFI_ARCH}.img
-options  gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}
+options  gpt loglevel=7 efivars.pstore_disable=1 efi_pstore.pstore_disable=1 efi_no_storage_paranoia add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}
 GUMEOF
 	
 	cat << GUMEOF > "${_ARCHBOOT_ISO_EXT_DIR}/loader/entries/archboot-${_UEFI_ARCH}-lts-efilinux.conf"
 title    Arch Linux LTS ${_UEFI_ARCH} Archboot via EFILINUX
-linux    /EFI/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi
-initrd   /boot/initramfs_${_UEFI_ARCH}.img
-options  -f \\boot\\vmlinuz_${_UEFI_ARCH}_lts gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}
+efi      /EFI/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi
 GUMEOF
 	
 	cat << GUMEOF > "${_ARCHBOOT_ISO_EXT_DIR}/loader/entries/uefi-shell-${_UEFI_ARCH}-v2.conf"
@@ -341,8 +358,8 @@ GUMEOF
 	cp -f "/usr/lib/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi" "${_ARCHBOOT_ISO_EXT_DIR}/EFI/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi"
 	echo
 	
-	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/EFI/efilinux/efilinux_._cfg_"
--f \\boot\\vmlinuz_${_UEFI_ARCH}_lts gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH} initrd=\\boot\\initramfs_${_UEFI_ARCH}.img
+	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/EFI/efilinux/efilinux.cfg"
+-f \\boot\\vmlinuz_${_UEFI_ARCH}_lts gpt loglevel=7 efivars.pstore_disable=1 efi_pstore.pstore_disable=1 efi_no_storage_paranoia add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH} initrd=\\boot\\initramfs_${_UEFI_ARCH}.img
 EOF
 	echo
 	
@@ -393,7 +410,7 @@ menuentry "Arch Linux ${_UEFI_ARCH} Archboot" {
     icon /EFI/refind/icons/os_arch.icns
     loader /boot/vmlinuz_${_UEFI_ARCH}
     initrd /boot/initramfs_${_UEFI_ARCH}.img
-    options "gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}"
+    options "gpt loglevel=7 efivars.pstore_disable=1 efi_pstore.pstore_disable=1 efi_no_storage_paranoia add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}"
     ostype Linux
     graphics off
 }
@@ -401,8 +418,6 @@ menuentry "Arch Linux ${_UEFI_ARCH} Archboot" {
 menuentry "Arch Linux LTS ${_UEFI_ARCH} Archboot via EFILINUX" {
     icon /EFI/refind/icons/os_arch.icns
     loader /EFI/efilinux/efilinux${_SPEC_UEFI_ARCH}.efi
-    initrd /boot/initramfs_${_UEFI_ARCH}.img
-    options "-f \\boot\\vmlinuz_${_UEFI_ARCH}_lts gpt loglevel=7 add_efi_memmap none=UEFI_ARCH_${_UEFI_ARCH}"
     ostype Linux
     graphics off
 }
@@ -434,7 +449,8 @@ _remove_i686_iso_files() {
 }
 
 _remove_x86_64_iso_files() {
-	
+	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/EFI" || true
+        rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/loader" || true
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/vmlinuz_x86_64_lts" || true
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/vmlinuz_x86_64" || true
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/initramfs_x86_64.img" || true
@@ -443,6 +459,11 @@ _remove_x86_64_iso_files() {
 	
 }
 
+_remove_packages() {
+	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/packages" || true
+	echo
+}
+
 _update_arch_setup_initramfs() {
 	
 	_initramfs_ext="$(mktemp -d /tmp/${_initramfs_name}_ext.XXXXXXXXXX)"
@@ -497,10 +518,40 @@ _download_pkgs() {
 	
 }
 
+_update_cd_uefi() {
+        MOUNT_FSIMG=$(mktemp -d)
+
+	## get size of boot x86_64 files
+	BOOTSIZE=$(du -bc ${_ARCHBOOT_ISO_EXT_DIR}/{EFI,loader,boot/*x86_64*} | grep total | cut -f1)
+	IMGSZ=$(( (${BOOTSIZE}*102)/100/1024 + 1)) # image size in sectors
+
+	## Create cdefiboot.img
+	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}"/CDEFI/
+	dd if=/dev/zero of="${_ARCHBOOT_ISO_EXT_DIR}"/CDEFI/cdefiboot.img bs="${IMGSZ}" count=1024 
+	mkfs.vfat "${_ARCHBOOT_ISO_EXT_DIR}"/CDEFI/cdefiboot.img
+	LOOPDEV="$(losetup --find --show "${_ARCHBOOT_ISO_EXT_DIR}"/CDEFI/cdefiboot.img)"
+
+	## Mount cdefiboot.img
+	mount -t vfat -o rw,users "${LOOPDEV}" "${MOUNT_FSIMG}"
+
+	## Copy UEFI files fo cdefiboot.img
+	mkdir "${MOUNT_FSIMG}"/boot
+	cp -r "${_ARCHBOOT_ISO_EXT_DIR}"/{EFI,loader} "${MOUNT_FSIMG}"
+	cp "${_ARCHBOOT_ISO_EXT_DIR}"/boot/*x86_64* "${MOUNT_FSIMG}"/boot
+
+	## Unmount cdefiboot.img
+	umount "${LOOPDEV}"
+	losetup --detach "${LOOPDEV}"
+	rm -rf "${MOUNT_FSIMG}"
+	_CD_UEFI_PARAMETERS="-eltorito-alt-boot -e CDEFI/cdefiboot.img -isohybrid-gpt-basdat -no-emul-boot"
+}
+
 [[ "${_REMOVE_i686}" == "1" ]] && _remove_i686_iso_files
 
 [[ "${_REMOVE_x86_64}" == "1" ]] && _remove_x86_64_iso_files
 
+[[ "${_REMOVE_PACKAGES}" == "1" ]] && _remove_packages
+
 if [[ "${_UPDATE_SETUP}" == "1" ]] && [[ -e "${_ARCHBOOT_ISO_WD}/setup" ]]; then
 	cd "${_ARCHBOOT_ISO_WD}/"
 	
@@ -534,10 +585,13 @@ fi
 
 [[ "${_UPDATE_SYSLINUX_CONFIG}" == "1" ]] && _update_syslinux_iso_config
 
+[[ "${_UPDATE_CD_UEFI}" == "1" ]] && _update_cd_uefi
+
 cd "${_ARCHBOOT_ISO_WD}/"
 
 ## Generate the BIOS+ISOHYBRID CD image using xorriso (extra/libisoburn package) in mkisofs emulation mode
 echo "Generating the modified ISO ..."
+
 xorriso -as mkisofs \
 	-iso-level 3 -rock -joliet \
 	-max-iso9660-filenames -omit-period \
@@ -549,6 +603,7 @@ xorriso -as mkisofs \
 	-eltorito-catalog boot/syslinux/boot.cat \
 	-no-emul-boot -boot-load-size 4 -boot-info-table \
 	-isohybrid-mbr /usr/lib/syslinux/isohdpfx.bin \
+	${_CD_UEFI_PARAMETERS} \
 	-output "${_ARCHBOOT_ISO_UPDATED_PATH}" "${_ARCHBOOT_ISO_EXT_DIR}/" &> "/tmp/archboot_update_xorriso.log"
 echo
 
@@ -572,12 +627,14 @@ unset _UEFI_ARCH
 unset _SPEC_UEFI_ARCH
 unset _REMOVE_i686
 unset _REMOVE_x86_64
+unset _REMOVE_PACKAGES
 unset _UPDATE_SETUP
 unset _UPDATE_UEFI_SHELL
 unset _UPDATE_UEFI_GUMMIBOOT
 unset _UPDATE_UEFI_REFIND
 unset _UPDATE_SYSLINUX
 unset _UPDATE_SYSLINUX_CONFIG
+unset _CD_UEFI_PARAMETERS
 unset _ARCHBOOT_ISO_OLD_PATH
 unset _ARCHBOOT_ISO_WD
 unset _ARCHBOOT_ISO_OLD_NAME
